EnchantList = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceHook-2.1")
local L = AceLibrary("AceLocale-2.2"):new("EnchantList")
local chatprefix = "!enchant"
local LOCKED=0

function EnchantList:OnInitialize()
	self:Reset()
	EnchantList:RegisterDB("EnchantListDB")
	EnchantList:RegisterDefaults("account", {
		enchants = {},
		enchantsname= {},
		enchantsdesc={},
		showincmessages=false,
		showexmessages=false,
		searchdesc=true,
		autoscan=true,
		TIMELOCK=10,
		chatprefix2="eList>",
		maxresults=10,
		}
	)  
	
local options = { 
    type="group",
    args = {
    	inc= {
    		type="toggle",
    		name=L["Show incoming requests"],
    		desc=L["Show incoming requests"],
    		get=function() return self.db.account.showincmessages end,
    		set=function(v) self.db.account.showincmessages = v end,
    		},
			out= {
				type="toggle",
				name=L["Show your replies"],
				desc=L["Show your replies"],
				get=function() return self.db.account.showexmessages end,
				set=function(v) self.db.account.showexmessages = v end,
			},
			autoscan= {
				type="toggle",
				name=L["Autoscan"],
				desc=L["Automatically scan your enchant panel when it opens"],
				get=function() return self.db.account.autoscan end,
				set=function(v) EnchantList:ToggleAutoScan(v) end,
			},
			desc= {
				type="toggle",
				name=L["Search description"],
				desc=L["Also search enchant description for a match"],
				get=function() return self.db.account.searchdesc end,
				set=function(v) self.db.account.searchdesc = v end,
			},
			scan= {
				type="execute",
				name=L["Manual scan (open your enchant panel first)"],
				desc=L["Manual scan (open your enchant panel first)"],
				func=function() EnchantList:Scan() end,
				order=101,
			},
			tlock= {
				type="range",
				name=L["Lock time between 2 requests (antispam)"],
				desc=L["Lock time between 2 requests (antispam)"],
				get=function() return self.db.account.TIMELOCK end,
    		set=function(v) self.db.account.TIMELOCK = v end,
				min=1,
				max=30,
				step=1,
			},
			prefix= {
				type="text",
				name=L["Prefix"],
				desc=L["Set the prefix for your answers (have to be specific enough)"],
				usage="...",
				get=function() return self.db.account.chatprefix2 end,
				set=function(v) if v=='' then self:Print(L["Can't be empty"]) else self.db.account.chatprefix2 = v end end,
				order=102,
			},
			maxresults= {
				type="range",
				name=L["Maximum results"],
				desc=L["Maximum of results sent to someone (0=no limit)"],
				get=function() return self.db.account.maxresults end,
				set=function(v) self.db.account.maxresults = v end,
				min=0,
				max=100,
				step=1,
			}				
		}
}

EnchantList:RegisterChatCommand({"/elist", "/enchantlist"}, options)
end

function EnchantList:OnEnable()
	self:Hook("ChatFrame_MessageEventHandler", "WhisperHandler", true)
	self:RegisterEvent("CHAT_MSG_WHISPER")
	if self.db.account.autoscan then self:RegisterEvent("CRAFT_SHOW") end
end

function EnchantList:OnDisable()
	self:Reset()
	self:UnhookAll()
		self:RegisterEvent("CHAT_MSG_WHISPER")
	self:UnregisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("CRAFT_SHOW")
	self:UnregisterEvent("CRAFT_SHOW")
end

function EnchantList:Reset()
	LOCKED=0
end

function EnchantList:WhisperHandler(event)
	if arg1 and arg1~="" then
		if (string.sub(arg1,1,string.len(chatprefix))==chatprefix and not self.db.account.showincmessages and event=="CHAT_MSG_WHISPER") then
			return
		end
		if ( string.sub(arg1,1,string.len(self.db.account.chatprefix2))==self.db.account.chatprefix2 and not self.db.account.showexmessages and event=="CHAT_MSG_WHISPER_INFORM") then
			return
		end
	end
	return self.hooks["ChatFrame_MessageEventHandler"](event)
end

function EnchantList:CHAT_MSG_WHISPER()
	local argum = self:GetArgs(arg1," ")
	local sender = arg2
	if argum[1]==chatprefix then 
		if arg6==L["AFK"] or arg6==L["DND"] then self:Respond(L["First clear your tag: "]..arg6,sender) return end
		table.remove(argum,1)
		argum=self:RecomposeLink(argum)
		if table.getn(argum)==0 then 
			self:Respond(L["Usage:"],sender) 
			self:Respond(L["!enchant <text1> <text2> ... (ie. !enchant boots agi, !enchant crusader,... )"],sender) 
			self:Respond(L["!enchant <itemlink> (ie. !enchant [Ashbringer])"],sender) 
			self:Respond(L["!enchant <itemlink> <text> (ie. !enchant [Ashbringer] agi)"],sender) 
			if self.db.account.searchdesc then self:Respond(L["Searching enchant description is currently enabled."],sender)
			else self:Respond(L["Searching enchant description is currently disabled."],sender)
			end
			return 
		end
		for k,v in pairs(argum) do
			if strfind(argum[k],"|cff") and strfind(argum[k],"|h|r") then
				local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount, iel = GetItemInfo(argum[k])
				if sType == L["Armor"] then 
					local b=0
					if iel=="INVTYPE_CHEST" then b=L["Chest"]
					elseif iel=="INVTYPE_ROBE" then b=L["Chest"]
					elseif iel=="INVTYPE_FEET" then b=L["Boots"]
					elseif iel=="INVTYPE_WRIST" then b=L["Bracer"]
					elseif iel=="INVTYPE_HAND" then b=L["Gloves"]
					elseif iel=="INVTYPE_CLOAK" then b=L["Cloak"]
					elseif iel=="INVTYPE_SHIELD" then b=L["Shield"]
					elseif iel=="INVTYPE_FINGER" then b=L["Ring"]
					else self:Respond(L["Error! Cannot enchant that."],sender) return end
					argum[k]=b
				elseif sType==L["Weapon"] then 
					local b=0
					if iel=="INVTYPE_2HWEAPON" then b=L["Weapon"]
					elseif iel=="INVTYPE_WEAPONMAINHAND" then b=L["Enchant Weapon"]
					elseif iel=="INVTYPE_WEAPONOFFHAND" then b=L["Enchant Weapon"]
					elseif iel=="INVTYPE_WEAPON" then b=L["Enchant Weapon"]
					else self:Respond(L["Error! Cannot enchant that."],sender) return end
					argum[k]=b
				end
			end
		end
		self:Handle(argum,sender)
	end
end

function EnchantList:GetArgs(text, separator)
	local args = {}
	local i = 0
	for value in string.gmatch(text, "[^"..separator.."]+") do
		i = i + 1
		args[i] = value
	end	
	return args
end

function EnchantList:RecomposeLink(argument)
	local message=argument
	for k,v in pairs(message) do
		if strfind(v,"|cff") then
				if strfind(v,"|h|r") then return message
				else 
					message[k]=v.." "..message[k+1]
					table.remove(message,k+1)
					return self:RecomposeLink(message)
				end
		end
	end
	return message
end

function EnchantList:Handle(argum,sender)
	if table.getn(self.db.account.enchants)==0 then 
		self:Respond(L["Database is currently empty."],sender) 
		self:Print(sender..L[" wants to use EnchantList but the enchanting panel has not yet been opened."])
		return 
	end
	local searchresult={}
	for i=1,table.getn(self.db.account.enchants) do
		local paf=0
		local enchantname=string.lower(self.db.account.enchantsname[i])
		local enchantdesc=string.lower(self.db.account.enchantsdesc[i])
		for j=1,table.getn(argum) do
			if strfind(enchantname, string.lower(argum[j])) then paf=paf+1 
			elseif self.db.account.searchdesc and strfind(enchantdesc, string.lower(argum[j])) then paf=paf+1
			end
		end 
		if paf==table.getn(argum) then table.insert(searchresult,self.db.account.enchants[i]) end	
	end
	if table.getn(searchresult)==0 then self:Respond(L["Nothing found."],sender)
	else 
		if LOCKED==0 then
			LOCKED=1
			if table.getn(searchresult) <= self.db.account.maxresults or self.db.account.maxresults == 0 then 
				for i=1,table.getn(searchresult) do 
					self:Respond(searchresult[i],sender) 
				end 
				self:Respond(table.getn(searchresult).." "..L["echantment(s) found."],sender)
			else 
				for i=1,self.db.account.maxresults do 
					self:Respond(searchresult[i],sender) 
				end 
			self:Respond(L["Maximum results exceeded, be more specific."].." "..self.db.account.maxresults.."/"..table.getn(searchresult),sender)
			end
			self:ScheduleEvent(function() LOCKED=0 return end, self.db.account.TIMELOCK)
		else self:Respond(L["Wait ... ("]..self.db.account.TIMELOCK..L[" seconds lock (antispam))"],sender)
		end
	end
end

function EnchantList:Respond(message,sender)
	SendChatMessage(self.db.account.chatprefix2.." "..message, "WHISPER", nil, sender)
end

function EnchantList:CRAFT_SHOW()
	if self.db.account.autoscan then 
		if not CraftFrameTitleText then return end
		if CraftFrameTitleText:GetText() ~= L["Enchanting"] then return end
		self:Scan()
	end
end

function EnchantList:ToggleAutoScan(v)
	self.db.account.autoscan = v 
	self:RegisterEvent("CRAFT_SHOW")
	if not v then self:UnregisterEvent("CRAFT_SHOW") end
end

function EnchantList:Scan()
	--if table.getn(self.enchants)~=0 then return end
	if not CraftFrameTitleText then self:Print(L["Open your enchant panel first"]) return end
	if CraftFrameTitleText:GetText() ~= L["Enchanting"] then self:Print(L["Open your enchant panel first"]) return end
	self.db.account.enchants={}
	self.db.account.enchantsname={}
	self.db.account.enchantsdesc={}
	for i=1,GetNumCrafts() do
		local skillname, craftSubSpellName, skilltype = GetCraftInfo(i)
		if skilltype~="header" and skillname then
				local link = GetCraftItemLink(i)
				local desc = GetCraftDescription(i)
				table.insert(self.db.account.enchants,link)
				table.insert(self.db.account.enchantsname,skillname)
			  if desc then table.insert(self.db.account.enchantsdesc,desc) else table.insert(self.db.account.enchantsdesc,"") end
		end	
	end
	return
end

